<?php

/* find < followed by any whitespace character any number of times */
$white_rx = "/<\s*/";
/* find < followed by any character except > any number of times, terminated by > */
$tag_rx = "/<[^>]*>/";

/* a demonstration array of tag match regular expressions */
$safelist_review = Array(
	/* find <p> or </p> */
	'/<p>|<\/p>/i',
	/* find <h0>..<h9> followed by any character except < any number of times, terminated by </h0>..</h9> */
	'/<h[0-9]>[^<]*<\/h[0-9]>/i',
	/* find <a href= followed by any character except > any number of times, terminated by > or </a> */
	'/<a\shref=.[\/h#][^>]*>|<\/a>/i',
	/* find <b> or </b> */
	'/<b>|<\/b>/i',
	'/<strong>|<\/strong>/i',
	'/<i>|<\/i>/i',
	'/<em>|<\/em>/i',
	/* find <br followed by any character except >, terminated by > */
	'/<br[^>]*>/i',
	/* find <img src= followed by any character once, followed by h followed by any character except > any number of times followed by /> */
	'/<img\ssrc=.[\/h][^>]*\/>|<img\ssrc="[\/h][^>]*>[^<]*<\/img>/i',
	/* find <hr followed by any character except > any number of times, terminated by > */
	'/<hr[^>]*>/',
	'/<ol[^>]*>|<\/ol[^>]*>/i',
	'/<ul[^>]*>|<\/ul[^>]*>/i',
	'/<li[^>]*>|<\/li[^>]*>/i',
	'/<code[^>]*>|<\/code[^>]*>/i',
	'/<pre[^>]*>|<\/pre[^>]*>/i'
);

/* a demonstration array of tag match regular expressions */
$safelist_comment = Array(
	'/<b>|<\/b>/i',
	'/<strong>|<\/strong>/i',
	'/<i>|<\/i>/i',
	'/<em>|<\/em>/i',
	'/<br[^>]*>/i'
);

/* a demonstration array of string/word match regular expressions */
$unsafe_words = Array(
	/* find javascript */
	"/javascript/e",
	/* find onclick */
	"/onclick/e",
	/* find onm followed by any alphanumeric character one or more times */
	"/onm\w{1,}=/e"
);

/* use $white_rx to trim whitespace tags e.g. <   tag> becomes <tag> */
function cleanWhitespace($input) {
	global $white_rx;
	$output = preg_replace($white_rx, "<", $input);
	return $output;
}			

/* returns all matches for $tag_rx as an array of strings*/
function getTags($input) {
	global $tag_rx;
	preg_match_all($tag_rx, $input, $matches);
	return $matches[0];
}

/* performs cleanWhitespace and then iterates over the results of getTags to HTML encode the non matching entities */
function replaceBadTags($input, $list) {
	/* clean whitespace */
	$output = cleanWhitespace($input);
	/* get tags */
	$finds = getTags($output);
	foreach($finds as $find) {
		/* assume we need to remove this tag */
		$clean = true;
		foreach($list as $safetag) {
			if(preg_match($safetag, $find) > 0) {
				/* the tag is in our safe list, we don't need to clean it */
				$clean = false;
				break;
			}
		}
		/* if we need to clean this tag, convert it to HTML encoded text */
		if($clean === true)
			$output = str_ireplace($find, htmlentities($find), $output);
	}
	return $output;
}

/* replace any matches between $unsafe_words and $input with "zilch" */
function replaceUnsafeWords($input) {
	global $unsafe_words;
	return preg_replace($unsafe_words, "zilch", $input);
}
?>